@@ -4,6 +4,7 @@
 #define TEMP_LRU 192
 
 #define CLEAR_LRU(id) (id & ~(3<<6))
+#define GET_LRU(id) (id & (3<<6))
 
 /* See items.c */
 uint64_t get_cas_id(void);
@@ -45,6 +46,14 @@ void item_stats_sizes_add(item *it);
 void item_stats_sizes_remove(item *it);
 bool item_stats_sizes_status(void);
 
+/* stats getter for slab automover */
+typedef struct {
+    int64_t evicted;
+    int64_t outofmemory;
+    uint32_t age;
+} item_stats_automove;
+void fill_item_stats_automove(item_stats_automove *am);
+
 item *do_item_get(const char *key, const size_t nkey, const uint32_t hv, conn *c, const bool do_update);
 item *do_item_touch(const char *key, const size_t nkey, uint32_t exptime, const uint32_t hv, conn *c);
 void item_stats_reset(void);
