@@ -19,6 +19,7 @@ enum log_entry_type {
     LOGGER_ITEM_GET,
     LOGGER_ITEM_STORE,
     LOGGER_CRAWLER_STATUS,
+    LOGGER_SLAB_MOVE,
 };
 
 enum log_entry_subtype {
@@ -92,7 +93,7 @@ typedef struct _logentry {
 #define LOG_MUTATIONS  (1<<3) /* set/append/incr/etc */
 #define LOG_SYSERRORS  (1<<4) /* malloc/etc errors */
 #define LOG_CONNEVENTS (1<<5) /* new client, closed, etc */
-#define LOG_EVICTIONS  (1<<6) /* defailts of evicted items */
+#define LOG_EVICTIONS  (1<<6) /* details of evicted items */
 #define LOG_STRICT     (1<<7) /* block worker instead of drop */
 #define LOG_RAWCMDS    (1<<9) /* raw ascii commands */
 
