@@ -375,6 +375,22 @@ static void do_slabs_free(void *ptr, const size_t size, unsigned int id) {
     return;
 }
 
+/* With refactoring of the various stats code the automover won't need a
+ * custom function here.
+ */
+void fill_slab_stats_automove(slab_stats_automove *am) {
+    int n;
+    pthread_mutex_lock(&slabs_lock);
+    for (n = 0; n < MAX_NUMBER_OF_SLAB_CLASSES; n++) {
+        slabclass_t *p = &slabclass[n];
+        slab_stats_automove *cur = &am[n];
+        cur->chunks_per_page = p->perslab;
+        cur->free_chunks = p->sl_curr;
+        cur->total_pages = p->slabs;
+    }
+    pthread_mutex_unlock(&slabs_lock);
+}
+
 static int nz_strcmp(int nzlength, const char *nz, const char *z) {
     int zlength=strlen(z);
     return (zlength == nzlength) && (strncmp(nz, z, zlength) == 0) ? 0 : -1;
@@ -516,7 +532,7 @@ static void memory_release() {
     while (mem_malloced > mem_limit &&
             (p = get_page_from_global_pool()) != NULL) {
         free(p);
-        mem_malloced -= settings.item_size_max;
+        mem_malloced -= settings.slab_page_size;
     }
 }
 
@@ -700,7 +716,7 @@ static void *slab_rebalance_alloc(const size_t size, unsigned int id) {
 }
 
 /* CALLED WITH slabs_lock HELD */
-/* detatches item/chunk from freelist. */
+/* detaches item/chunk from freelist. */
 static void slab_rebalance_cut_free(slabclass_t *s_cls, item *it) {
     /* Ensure this was on the freelist and nothing else. */
     assert(it->it_flags == ITEM_SLABBED);
@@ -716,6 +732,8 @@ enum move_status {
     MOVE_PASS=0, MOVE_FROM_SLAB, MOVE_FROM_LRU, MOVE_BUSY, MOVE_LOCKED
 };
 
+#define SLAB_MOVE_MAX_LOOPS 1000
+
 /* refcount == 0 is safe since nobody can incr while item_lock is held.
  * refcount != 0 is impossible since flags/etc can be modified in other
  * threads. instead, note we found a busy one and bail. logic in do_item_get
@@ -781,11 +799,12 @@ static int slab_rebalance_move(void) {
                 if ((hold_lock = item_trylock(hv)) == NULL) {
                     status = MOVE_LOCKED;
                 } else {
+                    bool is_linked = (it->it_flags & ITEM_LINKED);
                     refcount = refcount_incr(it);
                     if (refcount == 2) { /* item is linked but not busy */
                         /* Double check ITEM_LINKED flag here, since we're
                          * past a memory barrier from the mutex. */
-                        if ((it->it_flags & ITEM_LINKED) != 0) {
+                        if (is_linked) {
                             status = MOVE_FROM_LRU;
                         } else {
                             /* refcount == 1 + !ITEM_LINKED means the item is being
@@ -793,6 +812,16 @@ static int slab_rebalance_move(void) {
                              * yet. Let it bleed off on its own and try again later */
                             status = MOVE_BUSY;
                         }
+                    } else if (refcount > 2 && is_linked) {
+                        // TODO: Mark items for delete/rescue and process
+                        // outside of the main loop.
+                        if (slab_rebal.busy_loops > SLAB_MOVE_MAX_LOOPS) {
+                            slab_rebal.busy_deletes++;
+                            // Only safe to hold slabs lock because refcount
+                            // can't drop to 0 until we release item lock.
+                            do_item_unlink(it, hv);
+                        }
+                        status = MOVE_BUSY;
                     } else {
                         if (settings.verbose > 2) {
                             fprintf(stderr, "Slab reassign hit a busy item: refcount: %d (%d -> %d)\n",
@@ -941,6 +970,7 @@ static int slab_rebalance_move(void) {
             stats.slab_reassign_busy_items += slab_rebal.busy_items;
             STATS_UNLOCK();
             slab_rebal.busy_items = 0;
+            slab_rebal.busy_loops++;
         } else {
             slab_rebal.done++;
         }
@@ -959,6 +989,7 @@ static void slab_rebalance_finish(void) {
     uint32_t evictions_nomem;
     uint32_t inline_reclaim;
     uint32_t chunk_rescues;
+    uint32_t busy_deletes;
 
     pthread_mutex_lock(&slabs_lock);
 
@@ -991,7 +1022,7 @@ static void slab_rebalance_finish(void) {
     d_cls->slab_list[d_cls->slabs++] = slab_rebal.slab_start;
     /* Don't need to split the page into chunks if we're just storing it */
     if (slab_rebal.d_clsid > SLAB_GLOBAL_PAGE_POOL) {
-        memset(slab_rebal.slab_start, 0, (size_t)settings.item_size_max);
+        memset(slab_rebal.slab_start, 0, (size_t)settings.slab_page_size);
         split_slab_page_into_freelist(slab_rebal.slab_start,
             slab_rebal.d_clsid);
     } else if (slab_rebal.d_clsid == SLAB_GLOBAL_PAGE_POOL) {
@@ -999,6 +1030,7 @@ static void slab_rebalance_finish(void) {
         memory_release();
     }
 
+    slab_rebal.busy_loops = 0;
     slab_rebal.done       = 0;
     slab_rebal.s_clsid    = 0;
     slab_rebal.d_clsid    = 0;
@@ -1009,9 +1041,12 @@ static void slab_rebalance_finish(void) {
     inline_reclaim = slab_rebal.inline_reclaim;
     rescues   = slab_rebal.rescues;
     chunk_rescues = slab_rebal.chunk_rescues;
+    busy_deletes = slab_rebal.busy_deletes;
     slab_rebal.evictions_nomem    = 0;
     slab_rebal.inline_reclaim = 0;
     slab_rebal.rescues  = 0;
+    slab_rebal.chunk_rescues = 0;
+    slab_rebal.busy_deletes = 0;
 
     slab_rebalance_signal = 0;
 
@@ -1023,6 +1058,7 @@ static void slab_rebalance_finish(void) {
     stats.slab_reassign_evictions_nomem += evictions_nomem;
     stats.slab_reassign_inline_reclaim += inline_reclaim;
     stats.slab_reassign_chunk_rescues += chunk_rescues;
+    stats.slab_reassign_busy_deletes += busy_deletes;
     stats_state.slab_reassign_running = false;
     STATS_UNLOCK();
 
@@ -1042,7 +1078,7 @@ static void *slab_rebalance_thread(void *arg) {
     while (do_run_slab_rebalance_thread) {
         if (slab_rebalance_signal == 1) {
             if (slab_rebalance_start() < 0) {
-                /* Handle errors with more specifity as required. */
+                /* Handle errors with more specificity as required. */
                 slab_rebalance_signal = 0;
             }
 
@@ -1056,7 +1092,7 @@ static void *slab_rebalance_thread(void *arg) {
         } else if (was_busy) {
             /* Stuck waiting for some items to unlock, so slow down a bit
              * to give them a chance to free up */
-            usleep(50);
+            usleep(1000);
         }
 
         if (slab_rebalance_signal == 0) {
@@ -1088,6 +1124,7 @@ static int slabs_reassign_pick_any(int dst) {
 }
 
 static enum reassign_result_type do_slabs_reassign(int src, int dst) {
+    bool nospare = false;
     if (slab_rebalance_signal != 0)
         return REASSIGN_RUNNING;
 
@@ -1104,7 +1141,11 @@ static enum reassign_result_type do_slabs_reassign(int src, int dst) {
         dst < SLAB_GLOBAL_PAGE_POOL || dst > power_largest)
         return REASSIGN_BADCLASS;
 
+    pthread_mutex_lock(&slabs_lock);
     if (slabclass[src].slabs < 2)
+        nospare = true;
+    pthread_mutex_unlock(&slabs_lock);
+    if (nospare)
         return REASSIGN_NOSPARE;
 
     slab_rebal.s_clsid = src;
@@ -1150,7 +1191,7 @@ int start_slab_maintenance_thread(void) {
     }
 
     if (pthread_cond_init(&slab_rebalance_cond, NULL) != 0) {
-        fprintf(stderr, "Can't intiialize rebalance condition\n");
+        fprintf(stderr, "Can't initialize rebalance condition\n");
         return -1;
     }
     pthread_mutex_init(&slabs_rebalance_lock, NULL);
