@@ -11,9 +11,10 @@ use vars qw(@EXPORT);
 use Cwd;
 my $builddir = getcwd;
 
+my @unixsockets = ();
 
 @EXPORT = qw(new_memcached sleep mem_get_is mem_gets mem_gets_is mem_stats
-             supports_sasl free_port);
+             supports_sasl free_port supports_drop_priv);
 
 sub sleep {
     my $n = shift;
@@ -148,9 +149,15 @@ sub supports_sasl {
     return 0;
 }
 
+sub supports_drop_priv {
+    my $output = `$builddir/memcached-debug -h`;
+    return 1 if $output =~ /no_drop_privileges/i;
+    return 0;
+}
+
 sub new_memcached {
     my ($args, $passed_port) = @_;
-    my $port = $passed_port || free_port();
+    my $port = $passed_port;
     my $host = '127.0.0.1';
 
     if ($ENV{T_MEMD_USE_DAEMON}) {
@@ -164,14 +171,25 @@ sub new_memcached {
         croak("Failed to connect to specified memcached server.") unless $conn;
     }
 
-    my $udpport = free_port("udp");
-    $args .= " -p $port";
-    if (supports_udp()) {
-        $args .= " -U $udpport";
-    }
     if ($< == 0) {
         $args .= " -u root";
     }
+    $args .= " -o relaxed_privileges";
+
+    my $udpport;
+    if ($args =~ /-l (\S+)/) {
+        $port = free_port();
+        $udpport = free_port("udp");
+        $args .= " -p $port";
+        if (supports_udp()) {
+            $args .= " -U $udpport";
+        }
+    } elsif ($args !~ /-s (\S+)/) {
+        my $num = @unixsockets;
+        my $file = "/tmp/memcachetest.$$.$num";
+        $args .= " -s $file";
+        push(@unixsockets, $file);
+    }
 
     my $childpid = fork();
 
@@ -215,6 +233,12 @@ sub new_memcached {
     croak("Failed to startup/connect to memcached server.");
 }
 
+END {
+    for (@unixsockets) {
+        unlink $_;
+    }
+}
+
 ############################################################################
 package Memcached::Handle;
 sub new {
